@@ -1,8 +1,12 @@
 # -*- coding: utf-8 -*-
 
 
-from ocp_resources.constants import PROTOCOL_ERROR_EXCEPTION_DICT, TIMEOUT_4MINUTES
-from ocp_resources.resource import TIMEOUT, NamespacedResource
+from ocp_resources.constants import (
+    DEFAULT_CLUSTER_RETRY_EXCEPTIONS,
+    PROTOCOL_ERROR_EXCEPTION_DICT,
+    TIMEOUT_4MINUTES,
+)
+from ocp_resources.resource import NamespacedResource
 from ocp_resources.utils import TimeoutSampler
 from ocp_resources.virtual_machine_instance import VirtualMachineInstance
 
@@ -29,6 +33,12 @@ class VirtualMachine(NamespacedResource):
         STOPPED = "Stopped"
         STOPPING = "Stopping"
         WAITING_FOR_VOLUME_BINDING = "WaitingForVolumeBinding"
+        ERROR_UNSCHEDULABLE = "ErrorUnschedulable"
+        DATAVOLUME_ERROR = "DataVolumeError"
+        ERROR_PVC_NOT_FOUND = "ErrorPvcNotFound"
+        IMAGE_PULL_BACK_OFF = "ImagePullBackOff"
+        ERR_IMAGE_PULL = "ErrImagePull"
+        CRASH_LOOPBACK_OFF = "CrashLoopBackOff"
 
     def __init__(
         self,
@@ -39,6 +49,8 @@ class VirtualMachine(NamespacedResource):
         teardown=True,
         privileged_client=None,
         yaml_file=None,
+        delete_timeout=TIMEOUT_4MINUTES,
+        **kwargs,
     ):
         super().__init__(
             name=name,
@@ -47,6 +59,8 @@ class VirtualMachine(NamespacedResource):
             teardown=teardown,
             privileged_client=privileged_client,
             yaml_file=yaml_file,
+            delete_timeout=delete_timeout,
+            **kwargs,
         )
         self.body = body
 
@@ -64,37 +78,36 @@ class VirtualMachine(NamespacedResource):
         )
 
     def to_dict(self):
-        res = super().to_dict()
-        if self.yaml_file:
-            return res
+        super().to_dict()
+        if not self.yaml_file:
+            body_spec = self.body.get("spec") if self.body else None
+            self.res["spec"] = body_spec or {"template": {"spec": {}}}
 
-        body_spec = self.body.get("spec") if self.body else None
-        res["spec"] = body_spec or {"template": {"spec": {}}}
-        return res
-
-    def start(self, timeout=TIMEOUT, wait=False):
+    def start(self, timeout=TIMEOUT_4MINUTES, wait=False):
         self.api_request(method="PUT", action="start")
         if wait:
-            return self.wait_for_status(timeout=timeout, status=True)
+            return self.wait_for_ready_status(timeout=timeout, status=True)
 
-    def restart(self, timeout=TIMEOUT, wait=False):
+    def restart(self, timeout=TIMEOUT_4MINUTES, wait=False):
         self.api_request(method="PUT", action="restart")
         if wait:
             self.vmi.virt_launcher_pod.wait_deleted()
             return self.vmi.wait_until_running(timeout=timeout, stop_status="dummy")
 
-    def stop(self, timeout=TIMEOUT, wait=False):
+    def stop(
+        self, timeout=TIMEOUT_4MINUTES, vmi_delete_timeout=TIMEOUT_4MINUTES, wait=False
+    ):
         self.api_request(method="PUT", action="stop")
         if wait:
-            self.wait_for_status(timeout=timeout, status=None)
-            return self.vmi.wait_deleted()
+            self.wait_for_ready_status(timeout=timeout, status=None)
+            return self.vmi.wait_deleted(timeout=vmi_delete_timeout)
 
-    def wait_for_status(self, status, timeout=TIMEOUT, sleep=1):
+    def wait_for_ready_status(self, status, timeout=TIMEOUT_4MINUTES, sleep=1):
         """
-        Wait for resource to be in status
+        Wait for VM resource ready status to be at desire status
 
         Args:
-            status: Expected status: True for a running VM, None for a stopped VM.
+            status (any): True for a running VM, None for a stopped VM.
             timeout (int): Time to wait for the resource.
 
         Raises:
@@ -106,7 +119,10 @@ class VirtualMachine(NamespacedResource):
         samples = TimeoutSampler(
             wait_timeout=timeout,
             sleep=sleep,
-            exceptions_dict=PROTOCOL_ERROR_EXCEPTION_DICT,
+            exceptions_dict={
+                **PROTOCOL_ERROR_EXCEPTION_DICT,
+                **DEFAULT_CLUSTER_RETRY_EXCEPTIONS,
+            },
             func=self.api.get,
             field_selector=f"metadata.name=={self.name}",
             namespace=self.namespace,
